@@ -97,7 +97,7 @@
     _pD3D8Caps->PixelShaderVersion                = _pWineCaps->PixelShaderVersion; \
     _pD3D8Caps->MaxPixelShaderValue               = _pWineCaps->PixelShader1xMaxValue;
 
-void fixup_caps(WINED3DCAPS *pWineCaps);
+void fixup_caps(WINED3DCAPS *pWineCaps) DECLSPEC_HIDDEN;
 
 /* Direct3D8 Interfaces: */
 typedef struct IDirect3DBaseTexture8Impl IDirect3DBaseTexture8Impl;
@@ -112,14 +112,6 @@ typedef struct IDirect3DSwapChain8Impl IDirect3DSwapChain8Impl;
 typedef struct IDirect3DResource8Impl IDirect3DResource8Impl;
 typedef struct IDirect3DVolume8Impl IDirect3DVolume8Impl;
 typedef struct IDirect3DVertexBuffer8Impl IDirect3DVertexBuffer8Impl;
-
-/** Private Interfaces: */
-typedef struct IDirect3DStateBlockImpl IDirect3DStateBlockImpl;
-typedef struct IDirect3DVertexShaderImpl IDirect3DVertexShaderImpl;
-typedef struct IDirect3DPixelShaderImpl IDirect3DPixelShaderImpl;
-typedef struct IDirect3DVertexShaderDeclarationImpl IDirect3DVertexShaderDeclarationImpl;
-
-/* Advance declaration of structures to satisfy compiler */
 typedef struct IDirect3DVertexShader8Impl IDirect3DVertexShader8Impl;
 
 /* ===========================================================================
@@ -133,7 +125,7 @@ typedef struct IDirect3DVertexShader8Impl IDirect3DVertexShader8Impl;
 /*****************************************************************************
  * Predeclare the interface implementation structures
  */
-extern const IDirect3D8Vtbl Direct3D8_Vtbl;
+extern const IDirect3D8Vtbl Direct3D8_Vtbl DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3D implementation structure
@@ -148,16 +140,6 @@ struct IDirect3D8Impl
     IWineD3D               *WineD3D;
 };
 
-/* ---------------- */
-/* IDirect3DDevice8 */
-/* ---------------- */
-
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DDevice8Vtbl Direct3DDevice8_Vtbl;
-extern const IWineD3DDeviceParentVtbl d3d8_wined3d_device_parent_vtbl;
-
 /*****************************************************************************
  * IDirect3DDevice8 implementation structure
  */
@@ -211,6 +193,9 @@ struct IDirect3DDevice8Impl
     BOOL                          inDestruction;
 };
 
+HRESULT device_init(IDirect3DDevice8Impl *device, IWineD3D *wined3d, UINT adapter,
+        D3DDEVTYPE device_type, HWND focus_window, DWORD flags, D3DPRESENT_PARAMETERS *parameters) DECLSPEC_HIDDEN;
+
 /* ---------------- */
 /* IDirect3DVolume8 */
 /* ---------------- */
@@ -218,7 +203,6 @@ struct IDirect3DDevice8Impl
 /*****************************************************************************
  * IDirect3DVolume8 implementation structure
  */
-extern const IDirect3DVolume8Vtbl Direct3DVolume8_Vtbl;
 struct IDirect3DVolume8Impl
 {
     /* IUnknown fields */
@@ -235,6 +219,9 @@ struct IDirect3DVolume8Impl
     IUnknown                    *forwardReference;
 };
 
+HRESULT volume_init(IDirect3DVolume8Impl *volume, IDirect3DDevice8Impl *device, UINT width, UINT height,
+        UINT depth, DWORD usage, WINED3DFORMAT format, WINED3DPOOL pool) DECLSPEC_HIDDEN;
+
 /* ------------------- */
 /* IDirect3DSwapChain8 */
 /* ------------------- */
@@ -242,7 +229,7 @@ struct IDirect3DVolume8Impl
 /*****************************************************************************
  * Predeclare the interface implementation structures
  */
-extern const IDirect3DSwapChain8Vtbl Direct3DSwapChain8_Vtbl;
+extern const IDirect3DSwapChain8Vtbl Direct3DSwapChain8_Vtbl DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3DSwapChain8 implementation structure
@@ -264,11 +251,6 @@ struct IDirect3DSwapChain8Impl
 /* IDirect3DSurface8 */
 /* ----------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DSurface8Vtbl Direct3DSurface8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DSurface8 implementation structure
  */
@@ -289,11 +271,12 @@ struct IDirect3DSurface8Impl
 
     /* If set forward refcounting to this object */
     IUnknown                    *forwardReference;
-
-    /* Flags an implicit surface */
-    BOOL                        isImplicit;
 };
 
+HRESULT surface_init(IDirect3DSurface8Impl *surface, IDirect3DDevice8Impl *device,
+        UINT width, UINT height, D3DFORMAT format, BOOL lockable, BOOL discard, UINT level,
+        DWORD usage, D3DPOOL pool, D3DMULTISAMPLE_TYPE multisample_type, DWORD multisample_quality) DECLSPEC_HIDDEN;
+
 /* ------------------ */
 /* IDirect3DResource8 */
 /* ------------------ */
@@ -301,7 +284,7 @@ struct IDirect3DSurface8Impl
 /*****************************************************************************
  * Predeclare the interface implementation structures
  */
-extern const IDirect3DResource8Vtbl Direct3DResource8_Vtbl;
+extern const IDirect3DResource8Vtbl Direct3DResource8_Vtbl DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3DResource8 implementation structure
@@ -320,11 +303,6 @@ struct IDirect3DResource8Impl
 /* IDirect3DVertexBuffer8 */
 /* ---------------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DVertexBuffer8Vtbl Direct3DVertexBuffer8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DVertexBuffer8 implementation structure
  */
@@ -343,15 +321,13 @@ struct IDirect3DVertexBuffer8Impl
     DWORD                             fvf;
 };
 
+HRESULT vertexbuffer_init(IDirect3DVertexBuffer8Impl *buffer, IDirect3DDevice8Impl *device,
+        UINT size, DWORD usage, DWORD fvf, D3DPOOL pool) DECLSPEC_HIDDEN;
+
 /* --------------------- */
 /* IDirect3DIndexBuffer8 */
 /* --------------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DIndexBuffer8Vtbl Direct3DIndexBuffer8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DIndexBuffer8 implementation structure
  */
@@ -370,6 +346,9 @@ struct IDirect3DIndexBuffer8Impl
     WINED3DFORMAT                    format;
 };
 
+HRESULT indexbuffer_init(IDirect3DIndexBuffer8Impl *buffer, IDirect3DDevice8Impl *device,
+        UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
+
 /* --------------------- */
 /* IDirect3DBaseTexture8 */
 /* --------------------- */
@@ -391,11 +370,6 @@ struct IDirect3DBaseTexture8Impl
 /* IDirect3DCubeTexture8 */
 /* --------------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DCubeTexture8Vtbl Direct3DCubeTexture8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DCubeTexture8 implementation structure
  */
@@ -412,15 +386,13 @@ struct IDirect3DCubeTexture8Impl
     LPDIRECT3DDEVICE8                parentDevice;
 };
 
+HRESULT cubetexture_init(IDirect3DCubeTexture8Impl *texture, IDirect3DDevice8Impl *device,
+        UINT edge_length, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
+
 /* ----------------- */
 /* IDirect3DTexture8 */
 /* ----------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DTexture8Vtbl Direct3DTexture8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DTexture8 implementation structure
  */
@@ -437,15 +409,13 @@ struct IDirect3DTexture8Impl
     LPDIRECT3DDEVICE8            parentDevice;
 };
 
+HRESULT texture_init(IDirect3DTexture8Impl *texture, IDirect3DDevice8Impl *device,
+        UINT width, UINT height, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
+
 /* ----------------------- */
 /* IDirect3DVolumeTexture8 */
 /* ----------------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DVolumeTexture8Vtbl Direct3DVolumeTexture8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DVolumeTexture8 implementation structure
  */
@@ -462,14 +432,8 @@ struct IDirect3DVolumeTexture8Impl
     LPDIRECT3DDEVICE8                  parentDevice;
 };
 
-/* ----------------------- */
-/* IDirect3DStateBlockImpl */
-/* ----------------------- */
-
-/* TODO: Generate a valid GUIDs */
-/* {83B073CE-6F30-11d9-C687-00046142C14F} */
-DEFINE_GUID(IID_IDirect3DStateBlock8,
-0x83b073ce, 0x6f30, 0x11d9, 0xc6, 0x87, 0x0, 0x4, 0x61, 0x42, 0xc1, 0x4f);
+HRESULT volumetexture_init(IDirect3DVolumeTexture8Impl *texture, IDirect3DDevice8Impl *device,
+        UINT width, UINT height, UINT depth, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
 
 DEFINE_GUID(IID_IDirect3DVertexDeclaration8,
 0x5dd7478d, 0xcbf3, 0x41a6, 0x8c, 0xfd, 0xfd, 0x19, 0x2b, 0x11, 0xc7, 0x90);
@@ -480,50 +444,6 @@ DEFINE_GUID(IID_IDirect3DVertexShader8,
 DEFINE_GUID(IID_IDirect3DPixelShader8,
 0x6d3bdbdc, 0x5b02, 0x4415, 0xb8, 0x52, 0xce, 0x5e, 0x8b, 0xcc, 0xb2, 0x89);
 
-
-/*****************************************************************************
- * IDirect3DStateBlock8 interface
- */
-#define INTERFACE IDirect3DStateBlock8
-DECLARE_INTERFACE_(IDirect3DStateBlock8, IUnknown)
-{
-    /*** IUnknown methods ***/
-    STDMETHOD_(HRESULT,QueryInterface)(THIS_ REFIID riid, void** ppvObject) PURE;
-    STDMETHOD_(ULONG,AddRef)(THIS) PURE;
-    STDMETHOD_(ULONG,Release)(THIS) PURE;
-    /*** IDirect3DStateBlock9 methods ***/
-    STDMETHOD(GetDevice)(THIS_ struct IDirect3DDevice8** ppDevice) PURE;
-    STDMETHOD(Capture)(THIS) PURE;
-    STDMETHOD(Apply)(THIS) PURE;
-};
-#undef INTERFACE
-
-/*** IUnknown methods ***/
-#define IDirect3DStateBlock8_QueryInterface(p,a,b)  (p)->lpVtbl->QueryInterface(p,a,b)
-#define IDirect3DStateBlock8_AddRef(p)              (p)->lpVtbl->AddRef(p)
-#define IDirect3DStateBlock8_Release(p)             (p)->lpVtbl->Release(p)
-/*** IDirect3DStateBlock9 methods ***/
-#define IDirect3DStateBlock8_GetDevice(p,a)         (p)->lpVtbl->GetDevice(p,a)
-#define IDirect3DStateBlock8_Capture(p)             (p)->lpVtbl->Capture(p)
-#define IDirect3DStateBlock8_Apply(p)               (p)->lpVtbl->Apply(p)
-
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DStateBlock8Vtbl Direct3DStateBlock8_Vtbl;
-
-/*****************************************************************************
- * IDirect3DStateBlock implementation structure
- */
-typedef struct  IDirect3DStateBlock8Impl {
-    /* IUnknown fields */
-    const IDirect3DStateBlock8Vtbl *lpVtbl;
-    LONG                   ref;
-
-    /* IDirect3DResource8 fields */
-    IWineD3DStateBlock             *wineD3DStateBlock;
-} IDirect3DStateBlock8Impl;
-
 /*****************************************************************************
  * IDirect3DVertexDeclaration8 interface
  */
@@ -542,9 +462,6 @@ DECLARE_INTERFACE_(IDirect3DVertexDeclaration8, IUnknown)
 #define IDirect3DVertexDeclaration8_AddRef(p)              (p)->lpVtbl->AddRef(p)
 #define IDirect3DVertexDeclaration8_Release(p)             (p)->lpVtbl->Release(p)
 
-/*** Implementation ***/
-extern const IDirect3DVertexDeclaration8Vtbl Direct3DVertexDeclaration8_Vtbl;
-
 typedef struct {
     const IDirect3DVertexDeclaration8Vtbl *lpVtbl;
     LONG ref_count;
@@ -556,6 +473,10 @@ typedef struct {
     DWORD shader_handle;
 } IDirect3DVertexDeclaration8Impl;
 
+HRESULT vertexdeclaration_init(IDirect3DVertexDeclaration8Impl *declaration,
+        IDirect3DDevice8Impl *device, const DWORD *elements, DWORD shader_handle) DECLSPEC_HIDDEN;
+HRESULT vertexdeclaration_init_fvf(IDirect3DVertexDeclaration8Impl *declaration,
+        IDirect3DDevice8Impl *device, DWORD fvf) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3DVertexShader8 interface
@@ -597,11 +518,6 @@ DECLARE_INTERFACE_(IDirect3DPixelShader8,IUnknown)
 #define IDirect3DPixelShader8_AddRef(p)              (p)->lpVtbl->AddRef(p)
 #define IDirect3DPixelShader8_Release(p)             (p)->lpVtbl->Release(p)
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DVertexShader8Vtbl Direct3DVertexShader8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DVertexShader implementation structure
  */
@@ -614,19 +530,11 @@ struct IDirect3DVertexShader8Impl {
   IWineD3DVertexShader             *wineD3DVertexShader;
 };
 
+HRESULT vertexshader_init(IDirect3DVertexShader8Impl *shader, IDirect3DDevice8Impl *device,
+        const DWORD *declaration, const DWORD *byte_code, DWORD shader_handle, DWORD usage) DECLSPEC_HIDDEN;
+
 #define D3D8_MAX_VERTEX_SHADER_CONSTANTF 256
 
-
-/* ------------------------ */
-/* IDirect3DPixelShaderImpl */
-/* ------------------------ */
-
-
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DPixelShader8Vtbl Direct3DPixelShader8_Vtbl;
-
 /*****************************************************************************
  * IDirect3DPixelShader implementation structure
  */
@@ -638,22 +546,17 @@ typedef struct IDirect3DPixelShader8Impl {
     IWineD3DPixelShader             *wineD3DPixelShader;
 } IDirect3DPixelShader8Impl;
 
+HRESULT pixelshader_init(IDirect3DPixelShader8Impl *shader, IDirect3DDevice8Impl *device,
+        const DWORD *byte_code, DWORD shader_handle) DECLSPEC_HIDDEN;
+
 /**
  * Internals functions
  *
  * to see how not defined it here
  */
-D3DFORMAT d3dformat_from_wined3dformat(WINED3DFORMAT format);
-WINED3DFORMAT wined3dformat_from_d3dformat(D3DFORMAT format);
-void load_local_constants(const DWORD *d3d8_elements, IWineD3DVertexShader *wined3d_vertex_shader);
-UINT convert_to_wined3d_declaration(const DWORD *d3d8_elements, DWORD *d3d8_elements_size, WINED3DVERTEXELEMENT **wined3d_elements);
-size_t parse_token(const DWORD* pToken);
-
-/* Callbacks */
-extern ULONG WINAPI D3D8CB_DestroySwapChain (IWineD3DSwapChain *pSwapChain);
-extern ULONG WINAPI D3D8CB_DestroyDepthStencilSurface (IWineD3DSurface *pSurface);
-extern ULONG WINAPI D3D8CB_DestroyRenderTarget (IWineD3DSurface *pSurface);
-extern ULONG WINAPI D3D8CB_DestroySurface(IWineD3DSurface *pSurface);
-extern ULONG WINAPI D3D8CB_DestroyVolume(IWineD3DVolume *pVolume);
+D3DFORMAT d3dformat_from_wined3dformat(WINED3DFORMAT format) DECLSPEC_HIDDEN;
+WINED3DFORMAT wined3dformat_from_d3dformat(D3DFORMAT format) DECLSPEC_HIDDEN;
+void load_local_constants(const DWORD *d3d8_elements, IWineD3DVertexShader *wined3d_vertex_shader) DECLSPEC_HIDDEN;
+size_t parse_token(const DWORD *pToken) DECLSPEC_HIDDEN;
 
 #endif /* __WINE_D3DX8_PRIVATE_H */
