@@ -1974,7 +1974,7 @@ static void resource_test_data_init(IDirect3DDevice9 *device,
 
 static void resource_poison_data_init(struct resource_test_data *data, const struct resource_test_arg *arg)
 {
-    DWORD poison = 0xdeadbeef;
+    DWORD_PTR poison = 0xdeadbeef;
     unsigned int i;
 
     data->decl = (IDirect3DVertexDeclaration9 *)poison++;
