@@ -303,7 +303,7 @@ err_out:
  * Arguments, return values: See DDRAW_Create
  *
  ***********************************************************************/
-HRESULT WINAPI
+HRESULT WINAPI DECLSPEC_HOTPATCH
 DirectDrawCreate(GUID *GUID,
                  LPDIRECTDRAW *DD,
                  IUnknown *UnkOuter)
@@ -326,7 +326,7 @@ DirectDrawCreate(GUID *GUID,
  * Arguments, return values: See DDRAW_Create
  *
  ***********************************************************************/
-HRESULT WINAPI
+HRESULT WINAPI DECLSPEC_HOTPATCH
 DirectDrawCreateEx(GUID *GUID,
                    LPVOID *DD,
                    REFIID iid,
@@ -368,7 +368,7 @@ HRESULT WINAPI
 DirectDrawEnumerateA(LPDDENUMCALLBACKA Callback,
                      LPVOID Context)
 {
-    BOOL stop = FALSE;
+    TRACE("(%p, %p)\n", Callback, Context);
 
     TRACE(" Enumerating default DirectDraw HAL interface\n");
     /* We only have one driver */
@@ -377,11 +377,11 @@ DirectDrawEnumerateA(LPDDENUMCALLBACKA Callback,
         static CHAR driver_desc[] = "DirectDraw HAL",
         driver_name[] = "display";
 
-        stop = !Callback(NULL, driver_desc, driver_name, Context);
+        Callback(NULL, driver_desc, driver_name, Context);
     }
     __EXCEPT_PAGE_FAULT
     {
-        return E_INVALIDARG;
+        return DDERR_INVALIDPARAMS;
     }
     __ENDTRY
 
@@ -403,7 +403,16 @@ DirectDrawEnumerateExA(LPDDENUMCALLBACKEXA Callback,
                        LPVOID Context,
                        DWORD Flags)
 {
-    BOOL stop = FALSE;
+    TRACE("(%p, %p, 0x%08x)\n", Callback, Context, Flags);
+
+    if (Flags & ~(DDENUM_ATTACHEDSECONDARYDEVICES |
+                  DDENUM_DETACHEDSECONDARYDEVICES |
+                  DDENUM_NONDISPLAYDEVICES))
+        return DDERR_INVALIDPARAMS;
+
+    if (Flags)
+        FIXME("flags 0x%08x not handled\n", Flags);
+
     TRACE("Enumerating default DirectDraw HAL interface\n");
 
     /* We only have one driver by now */
@@ -413,11 +422,11 @@ DirectDrawEnumerateExA(LPDDENUMCALLBACKEXA Callback,
         driver_name[] = "display";
 
         /* QuickTime expects the description "DirectDraw HAL" */
-        stop = !Callback(NULL, driver_desc, driver_name, Context, 0);
+        Callback(NULL, driver_desc, driver_name, Context, 0);
     }
     __EXCEPT_PAGE_FAULT
     {
-        return E_INVALIDARG;
+        return DDERR_INVALIDPARAMS;
     }
     __ENDTRY;
 
@@ -428,22 +437,38 @@ DirectDrawEnumerateExA(LPDDENUMCALLBACKEXA Callback,
 /***********************************************************************
  * DirectDrawEnumerateW (DDRAW.@)
  *
- * Enumerates legacy drivers, unicode version. See
- * the comments above DirectDrawEnumerateA for more details.
- *
- * The Flag member is not supported right now.
+ * Enumerates legacy drivers, unicode version.
+ * This function is not implemented on Windows.
  *
  ***********************************************************************/
+HRESULT WINAPI
+DirectDrawEnumerateW(LPDDENUMCALLBACKW Callback,
+                     LPVOID Context)
+{
+    TRACE("(%p, %p)\n", Callback, Context);
+
+    if (!Callback)
+        return DDERR_INVALIDPARAMS;
+    else
+        return DDERR_UNSUPPORTED;
+}
 
 /***********************************************************************
  * DirectDrawEnumerateExW (DDRAW.@)
  *
- * Enumerates DirectDraw7 drivers, unicode version. See
- * the comments above DirectDrawEnumerateA for more details.
- *
- * The Flag member is not supported right now.
+ * Enumerates DirectDraw7 drivers, unicode version.
+ * This function is not implemented on Windows.
  *
  ***********************************************************************/
+HRESULT WINAPI
+DirectDrawEnumerateExW(LPDDENUMCALLBACKEXW Callback,
+                       LPVOID Context,
+                       DWORD Flags)
+{
+    TRACE("(%p, %p, 0x%x)\n", Callback, Context, Flags);
+
+    return DDERR_UNSUPPORTED;
+}
 
 /***********************************************************************
  * Classfactory implementation.
@@ -748,7 +773,6 @@ DestroyCallback(IDirectDrawSurface7 *surf,
                 void *context)
 {
     IDirectDrawSurfaceImpl *Impl = (IDirectDrawSurfaceImpl *)surf;
-    IDirectDrawImpl *ddraw = context;
     ULONG ref;
 
     ref = IDirectDrawSurface7_Release(surf);  /* For the EnumSurfaces */
@@ -760,9 +784,6 @@ DestroyCallback(IDirectDrawSurface7 *surf,
      */
     if( (!Impl->is_complex_root) || (Impl->first_attached != Impl) )
         return DDENUMRET_OK;
-    /* Skip our depth stencil surface, it will be released with the render target */
-    if( Impl == ddraw->DepthStencilBuffer)
-        return DDENUMRET_OK;
 
     /* Destroy the surface */
     while(ref) ref = IDirectDrawSurface7_Release(surf);
