@@ -324,7 +324,7 @@ static int dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARAM
             {
                 /* Already have X, need to queue it */
                 if (inst_id != -1)
-                    queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
+                    queue_event((LPDIRECTINPUTDEVICE8A)This, inst_id,
                                 wdata, GetCurrentTime(), This->base.dinput->evsequence);
                 inst_id = DIDFT_MAKEINSTANCE(WINE_MOUSE_Y_AXIS_INSTANCE) | DIDFT_RELAXIS;
                 wdata = pt1.y;
@@ -379,7 +379,7 @@ static int dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARAM
     if (inst_id != -1)
     {
         _dump_mouse_state(&This->m_state);
-        queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
+        queue_event((LPDIRECTINPUTDEVICE8A)This, inst_id,
                     wdata, GetCurrentTime(), This->base.dinput->evsequence++);
     }
 
