@@ -90,6 +90,7 @@ static REFIID tid_ids[] = {
     &DIID_DispHTMLDocument,
     &DIID_DispHTMLDOMTextNode,
     &DIID_DispHTMLElementCollection,
+    &DIID_DispHTMLFormElement,
     &DIID_DispHTMLGenericElement,
     &DIID_DispHTMLIFrame,
     &DIID_DispHTMLImg,
@@ -97,12 +98,14 @@ static REFIID tid_ids[] = {
     &DIID_DispHTMLLocation,
     &DIID_DispHTMLNavigator,
     &DIID_DispHTMLOptionElement,
+    &DIID_DispHTMLScreen,
     &DIID_DispHTMLSelectElement,
     &DIID_DispHTMLStyle,
     &DIID_DispHTMLTable,
     &DIID_DispHTMLTableRow,
     &DIID_DispHTMLUnknownElement,
     &DIID_DispHTMLWindow2,
+    &DIID_HTMLDocumentEvents,
     &IID_IHTMLAnchorElement,
     &IID_IHTMLBodyElement,
     &IID_IHTMLBodyElement2,
@@ -125,6 +128,7 @@ static REFIID tid_ids[] = {
     &IID_IHTMLElement4,
     &IID_IHTMLElementCollection,
     &IID_IHTMLEventObj,
+    &IID_IHTMLFormElement,
     &IID_IHTMLFrameBase,
     &IID_IHTMLFrameBase2,
     &IID_IHTMLGenericElement,
@@ -133,6 +137,7 @@ static REFIID tid_ids[] = {
     &IID_IHTMLInputElement,
     &IID_IHTMLLocation,
     &IID_IHTMLOptionElement,
+    &IID_IHTMLScreen,
     &IID_IHTMLSelectElement,
     &IID_IHTMLStyle,
     &IID_IHTMLStyle2,
@@ -144,6 +149,7 @@ static REFIID tid_ids[] = {
     &IID_IHTMLUniqueName,
     &IID_IHTMLWindow2,
     &IID_IHTMLWindow3,
+    &IID_IHTMLWindow4,
     &IID_IOmNavigator
 };
 
@@ -303,6 +309,61 @@ static dispex_data_t *preprocess_dispex_data(DispatchEx *This)
     return data;
 }
 
+static int id_cmp(const void *p1, const void *p2)
+{
+    return *(DISPID*)p1 - *(DISPID*)p2;
+}
+
+HRESULT get_dispids(tid_t tid, DWORD *ret_size, DISPID **ret)
+{
+    unsigned i, func_cnt;
+    FUNCDESC *funcdesc;
+    ITypeInfo *ti;
+    TYPEATTR *attr;
+    DISPID *ids;
+    HRESULT hres;
+
+    hres = get_typeinfo(tid, &ti);
+    if(FAILED(hres))
+        return hres;
+
+    hres = ITypeInfo_GetTypeAttr(ti, &attr);
+    if(FAILED(hres)) {
+        ITypeInfo_Release(ti);
+        return hres;
+    }
+
+    func_cnt = attr->cFuncs;
+    ITypeInfo_ReleaseTypeAttr(ti, attr);
+
+    ids = heap_alloc(func_cnt*sizeof(DISPID));
+    if(!ids) {
+        ITypeInfo_Release(ti);
+        return E_OUTOFMEMORY;
+    }
+
+    for(i=0; i < func_cnt; i++) {
+        hres = ITypeInfo_GetFuncDesc(ti, i, &funcdesc);
+        if(FAILED(hres))
+            break;
+
+        ids[i] = funcdesc->memid;
+        ITypeInfo_ReleaseFuncDesc(ti, funcdesc);
+    }
+
+    ITypeInfo_Release(ti);
+    if(FAILED(hres)) {
+        heap_free(ids);
+        return hres;
+    }
+
+    qsort(ids, func_cnt, sizeof(DISPID), id_cmp);
+
+    *ret_size = func_cnt;
+    *ret = ids;
+    return S_OK;
+}
+
 static CRITICAL_SECTION cs_dispex_static_data;
 static CRITICAL_SECTION_DEBUG cs_dispex_static_data_dbg =
 {
@@ -368,11 +429,13 @@ static inline dispex_dynamic_data_t *get_dynamic_data(DispatchEx *This, BOOL all
         : (This->dynamic_data = heap_alloc_zero(sizeof(dispex_dynamic_data_t)));
 }
 
-static HRESULT get_dynamic_prop(DispatchEx *This, const WCHAR *name, BOOL alloc, dynamic_prop_t **ret)
+static HRESULT get_dynamic_prop(DispatchEx *This, const WCHAR *name, DWORD flags, dynamic_prop_t **ret)
 {
-    dispex_dynamic_data_t *data = get_dynamic_data(This, alloc);
+    const BOOL alloc = flags & fdexNameEnsure;
+    dispex_dynamic_data_t *data;
     unsigned i;
 
+    data = get_dynamic_data(This, alloc);
     if(!data) {
         if(alloc)
             return E_OUTOFMEMORY;
@@ -382,12 +445,15 @@ static HRESULT get_dynamic_prop(DispatchEx *This, const WCHAR *name, BOOL alloc,
     }
 
     for(i=0; i < data->prop_cnt; i++) {
-        if(!strcmpW(data->props[i].name, name)) {
+        if(flags & fdexNameCaseInsensitive ? !strcmpiW(data->props[i].name, name) : !strcmpW(data->props[i].name, name)) {
             *ret = data->props+i;
             return S_OK;
         }
     }
 
+    if(!alloc)
+        return DISP_E_UNKNOWNNAME;
+
     TRACE("creating dynamic prop %s\n", debugstr_w(name));
 
     if(!data->buf_size) {
@@ -418,7 +484,7 @@ HRESULT dispex_get_dprop_ref(DispatchEx *This, const WCHAR *name, BOOL alloc, VA
     dynamic_prop_t *prop;
     HRESULT hres;
 
-    hres = get_dynamic_prop(This, name, alloc, &prop);
+    hres = get_dynamic_prop(This, name, alloc ? fdexNameEnsure : 0, &prop);
     if(FAILED(hres))
         return hres;
 
@@ -746,7 +812,7 @@ static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
 
     TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(bstrName), grfdex, pid);
 
-    if(grfdex & ~(fdexNameCaseSensitive|fdexNameEnsure|fdexNameImplicit|FDEX_VERSION_MASK))
+    if(grfdex & ~(fdexNameCaseSensitive|fdexNameCaseInsensitive|fdexNameEnsure|fdexNameImplicit|FDEX_VERSION_MASK))
         FIXME("Unsupported grfdex %x\n", grfdex);
 
     data = get_dispex_data(This);
@@ -782,7 +848,7 @@ static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
             return hres;
     }
 
-    hres = get_dynamic_prop(This, bstrName, grfdex&fdexNameEnsure, &dprop);
+    hres = get_dynamic_prop(This, bstrName, grfdex, &dprop);
     if(FAILED(hres))
         return hres;
 
@@ -870,7 +936,15 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
         }
         case DISPATCH_PROPERTYGET:
             return VariantCopy(pvarRes, var);
+        case DISPATCH_PROPERTYPUT|DISPATCH_PROPERTYPUTREF:
         case DISPATCH_PROPERTYPUT:
+            if(pdp->cArgs != 1 || (pdp->cNamedArgs == 1 && *pdp->rgdispidNamedArgs != DISPID_PROPERTYPUT)
+               || pdp->cNamedArgs > 1) {
+                FIXME("invalid args\n");
+                return E_INVALIDARG;
+            }
+
+            TRACE("put %s\n", debugstr_variant(pdp->rgvarg));
             VariantClear(var);
             return VariantCopy(var, pdp->rgvarg);
         default:
