@@ -437,8 +437,24 @@ static HRESULT WINAPI HTMLImgElement_put_name(IHTMLImgElement *iface, BSTR v)
 static HRESULT WINAPI HTMLImgElement_get_name(IHTMLImgElement *iface, BSTR *p)
 {
     HTMLImgElement *This = HTMLIMG_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString strName;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&strName, NULL);
+    nsres = nsIDOMHTMLImageElement_GetName(This->nsimg, &strName);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *str;
+
+        nsAString_GetData(&strName, &str);
+        *p = *str ? SysAllocString(str) : NULL;
+    }else {
+        ERR("GetName failed: %08x\n", nsres);
+    }
+    nsAString_Finish(&strName);
+
+    return NS_SUCCEEDED(nsres) ? S_OK : E_FAIL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
