@@ -113,6 +113,116 @@ static BOOL CALLBACK draw_state_callback( HDC hdc, LPARAM lparam, WPARAM wparam,
     return LOWORD(ret);
 }
 
+/* This function is a copy of the one in objects/font.c */
+static void logfont_32_to_16( const LOGFONTA* font32, LPLOGFONT16 font16 )
+{
+    font16->lfHeight = font32->lfHeight;
+    font16->lfWidth = font32->lfWidth;
+    font16->lfEscapement = font32->lfEscapement;
+    font16->lfOrientation = font32->lfOrientation;
+    font16->lfWeight = font32->lfWeight;
+    font16->lfItalic = font32->lfItalic;
+    font16->lfUnderline = font32->lfUnderline;
+    font16->lfStrikeOut = font32->lfStrikeOut;
+    font16->lfCharSet = font32->lfCharSet;
+    font16->lfOutPrecision = font32->lfOutPrecision;
+    font16->lfClipPrecision = font32->lfClipPrecision;
+    font16->lfQuality = font32->lfQuality;
+    font16->lfPitchAndFamily = font32->lfPitchAndFamily;
+    lstrcpynA( font16->lfFaceName, font32->lfFaceName, LF_FACESIZE );
+}
+
+
+/***********************************************************************
+ * Helper for wsprintf16
+ */
+
+#define WPRINTF_LEFTALIGN   0x0001  /* Align output on the left ('-' prefix) */
+#define WPRINTF_PREFIX_HEX  0x0002  /* Prefix hex with 0x ('#' prefix) */
+#define WPRINTF_ZEROPAD     0x0004  /* Pad with zeros ('0' prefix) */
+#define WPRINTF_LONG        0x0008  /* Long arg ('l' prefix) */
+#define WPRINTF_SHORT       0x0010  /* Short arg ('h' prefix) */
+#define WPRINTF_UPPER_HEX   0x0020  /* Upper-case hex ('X' specifier) */
+
+typedef enum
+{
+    WPR_UNKNOWN,
+    WPR_CHAR,
+    WPR_STRING,
+    WPR_SIGNED,
+    WPR_UNSIGNED,
+    WPR_HEXA
+} WPRINTF_TYPE;
+
+typedef struct
+{
+    UINT         flags;
+    UINT         width;
+    UINT         precision;
+    WPRINTF_TYPE type;
+} WPRINTF_FORMAT;
+
+static INT parse_format( LPCSTR format, WPRINTF_FORMAT *res )
+{
+    LPCSTR p = format;
+
+    res->flags = 0;
+    res->width = 0;
+    res->precision = 0;
+    if (*p == '-') { res->flags |= WPRINTF_LEFTALIGN; p++; }
+    if (*p == '#') { res->flags |= WPRINTF_PREFIX_HEX; p++; }
+    if (*p == '0') { res->flags |= WPRINTF_ZEROPAD; p++; }
+    while ((*p >= '0') && (*p <= '9'))  /* width field */
+    {
+        res->width = res->width * 10 + *p - '0';
+        p++;
+    }
+    if (*p == '.')  /* precision field */
+    {
+        p++;
+        while ((*p >= '0') && (*p <= '9'))
+        {
+            res->precision = res->precision * 10 + *p - '0';
+            p++;
+        }
+    }
+    if (*p == 'l') { res->flags |= WPRINTF_LONG; p++; }
+    else if (*p == 'h') { res->flags |= WPRINTF_SHORT; p++; }
+    switch(*p)
+    {
+    case 'c':
+    case 'C':  /* no Unicode in Win16 */
+        res->type = WPR_CHAR;
+        break;
+    case 's':
+    case 'S':
+        res->type = WPR_STRING;
+        break;
+    case 'd':
+    case 'i':
+        res->type = WPR_SIGNED;
+        break;
+    case 'u':
+        res->type = WPR_UNSIGNED;
+        break;
+    case 'p':
+        res->width = 8;
+        res->flags |= WPRINTF_ZEROPAD;
+        /* fall through */
+    case 'X':
+        res->flags |= WPRINTF_UPPER_HEX;
+        /* fall through */
+    case 'x':
+        res->type = WPR_HEXA;
+        break;
+    default: /* unknown format char */
+        res->type = WPR_UNKNOWN;
+        p--;  /* print format as normal char */
+        break;
+    }
+    return (INT)(p - format) + 1;
+}
+
 
 /**********************************************************************
  *		InitApp (USER.5)
@@ -172,6 +282,15 @@ BOOL16 WINAPI GetCursorPos16( POINT16 *pt )
 }
 
 
+/*******************************************************************
+ *		AnyPopup (USER.52)
+ */
+BOOL16 WINAPI AnyPopup16(void)
+{
+    return AnyPopup();
+}
+
+
 /***********************************************************************
  *		SetCursor (USER.69)
  */
@@ -492,6 +611,27 @@ INT16 WINAPI GetClipboardFormatName16( UINT16 id, LPSTR buffer, INT16 maxlen )
 }
 
 
+/**********************************************************************
+ *	    LoadMenu    (USER.150)
+ */
+HMENU16 WINAPI LoadMenu16( HINSTANCE16 instance, LPCSTR name )
+{
+    HRSRC16 hRsrc;
+    HGLOBAL16 handle;
+    HMENU16 hMenu;
+
+    if (HIWORD(name) && name[0] == '#') name = ULongToPtr(atoi( name + 1 ));
+    if (!name) return 0;
+
+    instance = GetExePtr( instance );
+    if (!(hRsrc = FindResource16( instance, name, (LPSTR)RT_MENU ))) return 0;
+    if (!(handle = LoadResource16( instance, hRsrc ))) return 0;
+    hMenu = LoadMenuIndirect16(LockResource16(handle));
+    FreeResource16( handle );
+    return hMenu;
+}
+
+
 /**********************************************************************
  *         CreateMenu    (USER.151)
  */
@@ -851,6 +991,80 @@ DWORD WINAPI UserSeeUserDo16(WORD wReqType, WORD wParam1, WORD wParam2, WORD wPa
 }
 
 
+/***********************************************************************
+ *           LookupMenuHandle   (USER.217)
+ */
+HMENU16 WINAPI LookupMenuHandle16( HMENU16 hmenu, INT16 id )
+{
+    FIXME( "%04x %04x: stub\n", hmenu, id );
+    return hmenu;
+}
+
+
+static LPCSTR parse_menu_resource( LPCSTR res, HMENU hMenu )
+{
+    WORD flags, id = 0;
+    LPCSTR str;
+    BOOL end_flag;
+
+    do
+    {
+        flags = GET_WORD(res);
+        end_flag = flags & MF_END;
+        /* Remove MF_END because it has the same value as MF_HILITE */
+        flags &= ~MF_END;
+        res += sizeof(WORD);
+        if (!(flags & MF_POPUP))
+        {
+            id = GET_WORD(res);
+            res += sizeof(WORD);
+        }
+        str = res;
+        res += strlen(str) + 1;
+        if (flags & MF_POPUP)
+        {
+            HMENU hSubMenu = CreatePopupMenu();
+            if (!hSubMenu) return NULL;
+            if (!(res = parse_menu_resource( res, hSubMenu ))) return NULL;
+            AppendMenuA( hMenu, flags, (UINT_PTR)hSubMenu, str );
+        }
+        else  /* Not a popup */
+        {
+            AppendMenuA( hMenu, flags, id, *str ? str : NULL );
+        }
+    } while (!end_flag);
+    return res;
+}
+
+/**********************************************************************
+ *	    LoadMenuIndirect    (USER.220)
+ */
+HMENU16 WINAPI LoadMenuIndirect16( LPCVOID template )
+{
+    HMENU hMenu;
+    WORD version, offset;
+    LPCSTR p = template;
+
+    TRACE("(%p)\n", template );
+    version = GET_WORD(p);
+    p += sizeof(WORD);
+    if (version)
+    {
+        WARN("version must be 0 for Win16\n" );
+        return 0;
+    }
+    offset = GET_WORD(p);
+    p += sizeof(WORD) + offset;
+    if (!(hMenu = CreateMenu())) return 0;
+    if (!parse_menu_resource( p, hMenu ))
+    {
+        DestroyMenu( hMenu );
+        return 0;
+    }
+    return HMENU_16(hMenu);
+}
+
+
 /*************************************************************************
  *		ScrollDC (USER.221)
  */
@@ -1369,6 +1583,7 @@ BOOL WINAPI DllEntryPoint( DWORD reason, HINSTANCE16 inst, WORD ds,
     if (USER_HeapSel) return TRUE;  /* already called */
 
     USER_HeapSel = ds;
+    register_wow_handlers();
     return TRUE;
 }
 
@@ -1555,6 +1770,122 @@ BOOL16 WINAPI SetMenuItemBitmaps16( HMENU16 hMenu, UINT16 nPos, UINT16 wFlags,
 }
 
 
+/***********************************************************************
+ *           wvsprintf   (USER.421)
+ */
+INT16 WINAPI wvsprintf16( LPSTR buffer, LPCSTR spec, VA_LIST16 args )
+{
+    WPRINTF_FORMAT format;
+    LPSTR p = buffer;
+    UINT i, len, sign;
+    CHAR number[20];
+    CHAR char_view = 0;
+    LPCSTR lpcstr_view = NULL;
+    INT int_view;
+    SEGPTR seg_str;
+
+    while (*spec)
+    {
+        if (*spec != '%') { *p++ = *spec++; continue; }
+        spec++;
+        if (*spec == '%') { *p++ = *spec++; continue; }
+        spec += parse_format( spec, &format );
+        switch(format.type)
+        {
+        case WPR_CHAR:
+            char_view = VA_ARG16( args, CHAR );
+            len = format.precision = 1;
+            break;
+        case WPR_STRING:
+            seg_str = VA_ARG16( args, SEGPTR );
+            if (IsBadReadPtr16( seg_str, 1 )) lpcstr_view = "";
+            else lpcstr_view = MapSL( seg_str );
+            if (!lpcstr_view) lpcstr_view = "(null)";
+            for (len = 0; !format.precision || (len < format.precision); len++)
+                if (!lpcstr_view[len]) break;
+            format.precision = len;
+            break;
+        case WPR_SIGNED:
+            if (format.flags & WPRINTF_LONG) int_view = VA_ARG16( args, INT );
+            else int_view = VA_ARG16( args, INT16 );
+            len = sprintf( number, "%d", int_view );
+            break;
+        case WPR_UNSIGNED:
+            if (format.flags & WPRINTF_LONG) int_view = VA_ARG16( args, UINT );
+            else int_view = VA_ARG16( args, UINT16 );
+            len = sprintf( number, "%u", int_view );
+            break;
+        case WPR_HEXA:
+            if (format.flags & WPRINTF_LONG) int_view = VA_ARG16( args, UINT );
+            else int_view = VA_ARG16( args, UINT16 );
+            len = sprintf( number, (format.flags & WPRINTF_UPPER_HEX) ? "%X" : "%x", int_view);
+            break;
+        case WPR_UNKNOWN:
+            continue;
+        }
+        if (format.precision < len) format.precision = len;
+        if (format.flags & WPRINTF_LEFTALIGN) format.flags &= ~WPRINTF_ZEROPAD;
+        if ((format.flags & WPRINTF_ZEROPAD) && (format.width > format.precision))
+            format.precision = format.width;
+        if (format.flags & WPRINTF_PREFIX_HEX) len += 2;
+
+        sign = 0;
+        if (!(format.flags & WPRINTF_LEFTALIGN))
+            for (i = format.precision; i < format.width; i++) *p++ = ' ';
+        switch(format.type)
+        {
+        case WPR_CHAR:
+            *p = char_view;
+            /* wsprintf16 ignores null characters */
+            if (*p != '\0') p++;
+            else if (format.width > 1) *p++ = ' ';
+            else len = 0;
+            break;
+        case WPR_STRING:
+            if (len) memcpy( p, lpcstr_view, len );
+            p += len;
+            break;
+        case WPR_HEXA:
+            if (format.flags & WPRINTF_PREFIX_HEX)
+            {
+                *p++ = '0';
+                *p++ = (format.flags & WPRINTF_UPPER_HEX) ? 'X' : 'x';
+                len -= 2;
+            }
+            /* fall through */
+        case WPR_SIGNED:
+            /* Transfer the sign now, just in case it will be zero-padded*/
+            if (number[0] == '-')
+            {
+                *p++ = '-';
+                sign = 1;
+            }
+            /* fall through */
+        case WPR_UNSIGNED:
+            for (i = len; i < format.precision; i++) *p++ = '0';
+            if (len > sign) memcpy( p, number + sign, len - sign );
+            p += len-sign;
+            break;
+        case WPR_UNKNOWN:
+            continue;
+        }
+        if (format.flags & WPRINTF_LEFTALIGN)
+            for (i = format.precision; i < format.width; i++) *p++ = ' ';
+    }
+    *p = 0;
+    return p - buffer;
+}
+
+
+/***********************************************************************
+ *           _wsprintf   (USER.420)
+ */
+INT16 WINAPIV wsprintf16( LPSTR buffer, LPCSTR spec, VA_LIST16 valist )
+{
+    return wvsprintf16( buffer, spec, valist );
+}
+
+
 /***********************************************************************
  *           lstrcmp   (USER.430)
  */
@@ -1908,6 +2239,118 @@ INT16 WINAPI GetKeyboardLayoutName16( LPSTR name )
 }
 
 
+/***********************************************************************
+ *		SystemParametersInfo (USER.483)
+ */
+BOOL16 WINAPI SystemParametersInfo16( UINT16 uAction, UINT16 uParam,
+                                      LPVOID lpvParam, UINT16 fuWinIni )
+{
+    BOOL16 ret;
+
+    TRACE("(%u, %u, %p, %u)\n", uAction, uParam, lpvParam, fuWinIni);
+
+    switch (uAction)
+    {
+    case SPI_GETBEEP:
+    case SPI_GETSCREENSAVEACTIVE:
+    case SPI_GETICONTITLEWRAP:
+    case SPI_GETMENUDROPALIGNMENT:
+    case SPI_GETFASTTASKSWITCH:
+    case SPI_GETDRAGFULLWINDOWS:
+    {
+	BOOL tmp;
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam ? &tmp : NULL, fuWinIni );
+	if (ret && lpvParam) *(BOOL16 *)lpvParam = tmp;
+	break;
+    }
+
+    case SPI_GETBORDER:
+    case SPI_ICONHORIZONTALSPACING:
+    case SPI_GETSCREENSAVETIMEOUT:
+    case SPI_GETGRIDGRANULARITY:
+    case SPI_GETKEYBOARDDELAY:
+    case SPI_ICONVERTICALSPACING:
+    {
+	INT tmp;
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam ? &tmp : NULL, fuWinIni );
+	if (ret && lpvParam) *(INT16 *)lpvParam = tmp;
+	break;
+    }
+
+    case SPI_GETKEYBOARDSPEED:
+    case SPI_GETMOUSEHOVERWIDTH:
+    case SPI_GETMOUSEHOVERHEIGHT:
+    case SPI_GETMOUSEHOVERTIME:
+    {
+	DWORD tmp;
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam ? &tmp : NULL, fuWinIni );
+	if (ret && lpvParam) *(WORD *)lpvParam = tmp;
+	break;
+    }
+
+    case SPI_GETICONTITLELOGFONT:
+    {
+	LOGFONTA tmp;
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam ? &tmp : NULL, fuWinIni );
+	if (ret && lpvParam) logfont_32_to_16( &tmp, (LPLOGFONT16)lpvParam );
+	break;
+    }
+
+    case SPI_GETNONCLIENTMETRICS:
+    {
+	NONCLIENTMETRICSA tmp;
+	LPNONCLIENTMETRICS16 lpnm16 = (LPNONCLIENTMETRICS16)lpvParam;
+	if (lpnm16 && lpnm16->cbSize == sizeof(NONCLIENTMETRICS16))
+	{
+	    tmp.cbSize = sizeof(NONCLIENTMETRICSA);
+	    ret = SystemParametersInfoA( uAction, uParam, &tmp, fuWinIni );
+	    if (ret)
+            {
+                lpnm16->iBorderWidth	 = tmp.iBorderWidth;
+                lpnm16->iScrollWidth	 = tmp.iScrollWidth;
+                lpnm16->iScrollHeight	 = tmp.iScrollHeight;
+                lpnm16->iCaptionWidth	 = tmp.iCaptionWidth;
+                lpnm16->iCaptionHeight	 = tmp.iCaptionHeight;
+                lpnm16->iSmCaptionWidth	 = tmp.iSmCaptionWidth;
+                lpnm16->iSmCaptionHeight = tmp.iSmCaptionHeight;
+                lpnm16->iMenuWidth       = tmp.iMenuWidth;
+                lpnm16->iMenuHeight      = tmp.iMenuHeight;
+                logfont_32_to_16( &tmp.lfCaptionFont,	&lpnm16->lfCaptionFont );
+                logfont_32_to_16( &tmp.lfSmCaptionFont,	&lpnm16->lfSmCaptionFont );
+                logfont_32_to_16( &tmp.lfMenuFont,	&lpnm16->lfMenuFont );
+                logfont_32_to_16( &tmp.lfStatusFont,	&lpnm16->lfStatusFont );
+                logfont_32_to_16( &tmp.lfMessageFont,	&lpnm16->lfMessageFont );
+            }
+	}
+	else /* winfile 95 sets cbSize to 340 */
+	    ret = SystemParametersInfoA( uAction, uParam, lpvParam, fuWinIni );
+	break;
+    }
+
+    case SPI_GETWORKAREA:
+    {
+	RECT tmp;
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam ? &tmp : NULL, fuWinIni );
+	if (ret && lpvParam)
+        {
+            RECT16 *r16 = lpvParam;
+            r16->left   = tmp.left;
+            r16->top    = tmp.top;
+            r16->right  = tmp.right;
+            r16->bottom = tmp.bottom;
+        }
+	break;
+    }
+
+    default:
+	ret = SystemParametersInfoA( uAction, uParam, lpvParam, fuWinIni );
+        break;
+    }
+
+    return ret;
+}
+
+
 /***********************************************************************
  *           FormatMessage   (USER.606)
  */
