@@ -476,11 +476,13 @@ static void set_focus( Display *display, HWND hwnd, Time time )
 {
     HWND focus;
     Window win;
+    GUITHREADINFO threadinfo;
 
     TRACE( "setting foreground window to %p\n", hwnd );
     SetForegroundWindow( hwnd );
 
-    focus = GetFocus();
+    GetGUIThreadInfo(0, &threadinfo);
+    focus = threadinfo.hwndFocus;
     if (focus) focus = GetAncestor( focus, GA_ROOT );
     win = X11DRV_get_whole_window(focus);
 
@@ -505,6 +507,14 @@ static void handle_wm_protocols( HWND hwnd, XClientMessageEvent *event )
 
     if (protocol == x11drv_atom(WM_DELETE_WINDOW))
     {
+        if (hwnd == GetDesktopWindow())
+        {
+            /* The desktop window does not have a close button that we can
+             * pretend to click. Therefore, we simply send it a close command. */
+            SendMessageW(hwnd, WM_SYSCOMMAND, SC_CLOSE, 0);
+            return;
+        }
+
         /* Ignore the delete window request if the window has been disabled
          * and we are in managed mode. This is to disallow applications from
          * being closed by the window manager while in a modal state.
@@ -676,6 +686,7 @@ static void X11DRV_FocusOut( HWND hwnd, XEvent *xev )
         wine_tsx11_unlock();
     }
     if (hwnd != GetForegroundWindow()) return;
+    if (root_window != DefaultRootWindow(event->display)) return;
     SendMessageW( hwnd, WM_CANCELMODE, 0, 0 );
 
     /* don't reset the foreground window, if the window which is
