@@ -2025,14 +2025,11 @@ void CDECL X11DRV_SetFocus( HWND hwnd )
     wine_tsx11_lock();
     changes.stack_mode = Above;
     XConfigureWindow( display, data->whole_window, CWStackMode, &changes );
-    if (root_window == DefaultRootWindow(display))
-    {
-        /* we must not use CurrentTime (ICCCM), so try to use last message time instead */
-        /* FIXME: this is not entirely correct */
-        XSetInputFocus( display, data->whole_window, RevertToParent,
-                        /* CurrentTime */
-                        GetMessageTime() - EVENT_x11_time_to_win32_time(0));
-    }
+    /* we must not use CurrentTime (ICCCM), so try to use last message time instead */
+    /* FIXME: this is not entirely correct */
+    XSetInputFocus( display, data->whole_window, RevertToParent,
+                    /* CurrentTime */
+                    GetMessageTime() - EVENT_x11_time_to_win32_time(0));
     wine_tsx11_unlock();
 }
 
@@ -2308,8 +2305,6 @@ LRESULT CDECL X11DRV_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
     {
     case WM_X11DRV_ACQUIRE_SELECTION:
         return X11DRV_AcquireClipboard( hwnd );
-    case WM_X11DRV_DELETE_WINDOW:
-        return SendMessageW( hwnd, WM_SYSCOMMAND, SC_CLOSE, 0 );
     case WM_X11DRV_SET_WIN_FORMAT:
         return set_win_format( hwnd, (XID)wp );
     case WM_X11DRV_SET_WIN_REGION:
