@@ -98,41 +98,12 @@ static	void WINMM_DeleteIData(void)
     DeleteCriticalSection(&WINMM_cs);
 }
 
-/******************************************************************
- *             WINMM_LoadMMSystem
- *
- */
-static HANDLE (WINAPI *pGetModuleHandle16)(LPCSTR);
-static DWORD (WINAPI *pLoadLibrary16)(LPCSTR);
-
-BOOL WINMM_CheckForMMSystem(void)
-{
-    /* 0 is not checked yet, -1 is not present, 1 is present */
-    static      int    loaded /* = 0 */;
-
-    if (loaded == 0)
-    {
-        HANDLE      h = GetModuleHandleA("kernel32");
-        loaded = -1;
-        if (h)
-        {
-            pGetModuleHandle16 = (void*)GetProcAddress(h, "GetModuleHandle16");
-            pLoadLibrary16 = (void*)GetProcAddress(h, (LPCSTR)35); /* ordinal for LoadLibrary16 */
-            if (pGetModuleHandle16 && pLoadLibrary16 &&
-                (pGetModuleHandle16("MMSYSTEM.DLL") || pLoadLibrary16("MMSYSTEM.DLL")))
-                loaded = 1;
-        }
-    }
-    return loaded > 0;
-}
-
 /******************************************************************
  *             WINMM_ErrorToString
  */
 const char* WINMM_ErrorToString(MMRESULT error)
 {
 #define ERR_TO_STR(dev) case dev: return #dev
-    static char unknown[32];
     switch (error) {
     ERR_TO_STR(MMSYSERR_NOERROR);
     ERR_TO_STR(MMSYSERR_ERROR);
@@ -160,9 +131,8 @@ const char* WINMM_ErrorToString(MMRESULT error)
     ERR_TO_STR(WAVERR_UNPREPARED);
     ERR_TO_STR(WAVERR_SYNC);
     }
-    sprintf(unknown, "Unknown(0x%08x)", error);
-    return unknown;
 #undef ERR_TO_STR
+    return wine_dbg_sprintf("Unknown(0x%08x)", error);
 }
 
 /**************************************************************************
